package com.engine.thinktrans.service.impl;

import com.engine.core.impl.Service;
import com.engine.thinktrans.entity.ReportChildrenData;
import com.engine.thinktrans.entity.ReportColumns;
import com.engine.thinktrans.entity.param.TendencyReportParam;
import com.engine.thinktrans.entity.vo.TendencyReportVO;
import com.engine.thinktrans.service.ReportCollectService;
import com.engine.thinktrans.util.CommonDateUtil;
import com.engine.thinktrans.util.OperateUtil;
import com.weaver.general.Util;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import weaver.conn.RecordSet;
import weaver.hrm.company.DepartmentComInfo;

import java.time.Year;
import java.time.YearMonth;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * @Author liang.cheng
 * @Date 2023/11/16 10:43 AM
 * @Description: TODO
 * @Version 1.0
 */
public class ReportCollectServiceImpl extends Service implements ReportCollectService {

    @Override
    public TendencyReportVO getPersonTendency(TendencyReportParam tendencyReportParam) {

        List<ReportColumns> columns = getReportColumns();
        List<Integer> deptList = new ArrayList<>();
        RecordSet rs = new RecordSet();
        rs.executeQuery("select a.deptid from hrmdepartmentdefined a left join hrmdepartment b on a.deptid =b.id where a.bmcj in (0,1)");
        while (rs.next()) {
            deptList.add(Util.getIntValue(rs.getString("deptid")));
        }
        String deptIds = tendencyReportParam.getDeptIds();
        if (StringUtils.isNotEmpty(deptIds)) {
            List<Integer> list = Arrays.stream(deptIds.split(","))
                    .map(Integer::parseInt)
                    .collect(Collectors.toList());
            deptList = list.stream()
                    .filter(deptList::contains)
                    .collect(Collectors.toList());
        }

        List<ReportChildrenData> datas = new ArrayList<>();
        List<List<Integer>> countLists = new ArrayList<>();

        deptList.forEach(id -> {
            List<Integer> countList = new ArrayList<>();
            ReportChildrenData childrenData = getReportChildrenData(tendencyReportParam,id,countList);
            datas.add(childrenData);
            countLists.add(countList);
        });

        //求和
        List<Integer> result = countLists.stream()
                .reduce((a, b) -> {
                    List<Integer> sum = new ArrayList<>();
                    for (int i = 0; i < a.size(); i++) {
                        sum.add(a.get(i) + b.get(i));
                    }
                    return sum;
                })
                .orElse(new ArrayList<>());

        List<String> stringList = result.stream()
                .map(String::valueOf)
                .collect(Collectors.toList());

        List<String> sums = new ArrayList<>();
        sums.add("总数");
        sums.addAll(stringList);

        datas.add(ReportChildrenData.builder().data(sums).build());
        return TendencyReportVO.builder()
                .columns(columns)
                .datas(datas)
                .build();
    }

    /**
     *
     * @param param
     * @return
     */
    @SneakyThrows
    private ReportChildrenData getReportChildrenData(TendencyReportParam param, Integer id,List<Integer> countList) {
        RecordSet rs = new RecordSet();
        DepartmentComInfo departmentComInfo = new DepartmentComInfo();
        List<String> data = new ArrayList<>();
        data.add(departmentComInfo.getDepartmentName(String.valueOf(id)));
        ArrayList<String> allSubDepartment = new ArrayList<>();
        departmentComInfo.getAllChildDeptByDepId(allSubDepartment, String.valueOf(id));
        allSubDepartment.add(String.valueOf(id));
        String allId = StringUtils.join(allSubDepartment, ",");

        if (param.getYear() == null) {
            param.setYear(Year.now().getValue());
        }

        String startDate = CommonDateUtil.getYearDayFirst(param.getYear());
        List<YearMonth> yearMonths = CommonDateUtil.getYearMonths(param.getYear());

        StringBuilder st = new StringBuilder();
        st.append("select count(1) as sum from hrmresource where status < 4  and companystartdate >= ? and companystartdate <= ? and departmentid in");
        st.append(" (").append(allId).append(")");
        if (StringUtils.isNotEmpty(param.getUseKind())) {
            st.append(" and usekind = ").append(param.getUseKind());
        }
        yearMonths.forEach(yearMonth -> {
            String endMonth = CommonDateUtil.getFormatYear(CommonDateUtil.toDateEndOfMonth(yearMonth));
            rs.executeQuery(st.toString(),startDate,endMonth);
            if (rs.next()) {
                Integer sum = Util.getIntValue(rs.getString("sum"),0);
                data.add(String.valueOf(sum));
                countList.add(sum);
            }
        });

        return ReportChildrenData.builder().data(data).build();
    }


    /**
     * 月份
     * @return
     */
    private List<ReportColumns> getReportColumns() {
        List<ReportColumns> columns = new ArrayList<>();
        columns.add(ReportColumns.builder().title("月份").key("0").build());
        columns.add(ReportColumns.builder().title("1月").key("1").build());
        columns.add(ReportColumns.builder().title("2月").key("2").build());
        columns.add(ReportColumns.builder().title("3月").key("3").build());
        columns.add(ReportColumns.builder().title("4月").key("4").build());
        columns.add(ReportColumns.builder().title("5月").key("5").build());
        columns.add(ReportColumns.builder().title("6月").key("6").build());
        columns.add(ReportColumns.builder().title("7月").key("7").build());
        columns.add(ReportColumns.builder().title("8月").key("8").build());
        columns.add(ReportColumns.builder().title("9月").key("9").build());
        columns.add(ReportColumns.builder().title("10月").key("10").build());
        columns.add(ReportColumns.builder().title("11月").key("11").build());
        columns.add(ReportColumns.builder().title("12月").key("12").build());
        return columns;
    }


}
